# Copyright (c) 2010, Daniel Bueltmann, All rights reserved.

# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:

# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS'
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import httplib
import urllib
import progressbar
import xspfparser
import os
import mutagen.easyid3

def toDict(data):
    r = {}
    for l in data.split("\n"):
        f = l.split("=")
        r[f[0]] = "=".join(f[1:])
    return r

def login(username, md5pass):

    conn = httplib.HTTPConnection("ws.audioscrobbler.com")
    uri = "/radio/handshake.php?username=%s&passwordmd5=%s" % (username, md5pass)

    conn.request("GET", uri)
    r1 = conn.getresponse()
    data = r1.read()

    return toDict(data)

def adjust(session, lastfmuri):

    conn = httplib.HTTPConnection("ws.audioscrobbler.com")
    uri = "/radio/adjust.php?session=%s&url=%s" % (session, lastfmuri)

    conn.request("GET", uri)
    r1 = conn.getresponse()
    data = r1.read()

    return toDict(data)

def playlist(session):
    conn = httplib.HTTPConnection("ws.audioscrobbler.com")
    uri = "/radio/xspf.php?sk=%s&discovery=0&desktop=1.5.1" % (session)

    #conn.request("GET", uri)
    #r1 = conn.getresponse()
    #data = r1.read()


    result = xspfparser.parse("http://ws.audioscrobbler.com"+uri)

    return result


#lastfm://user/$user/personal
#lastfm://user/$user/playlist
#lastfm://user/$user/loved
#lastfm://user/$username/neighbours
#lastfm://user/$username/recommended/100

#lastfm://usertags/$user/$tag
#lastfm://globaltags/$tag

#lastfm://artist/$artist
#lastfm://artist/$artist/similarartists
#lastfm://artist/$artist/fans

#lastfm://group/$groupname

#lastfm://play/tracks/$trackid,$trackid,$trackid

def artisturi(artist):
    uri = "lastfm://artist/%s" % urllib.quote(artist)
    return uri

def taguri(tag):
    uri = "lastfm://globaltags/%s" % urllib.quote(tag)
    return uri

class ProgressHook:

    def __init__(self):
        self.pbar = None

    def setText(self, text):
        print text

    def __call__(self, sofar, blocksize, size):
        if self.pbar is None:
            mbs = float(size) / float(1000000)

            self.widgets = ['%.2f Mb : ' %mbs, progressbar.Percentage(), ' ', progressbar.Bar(marker=progressbar.RotatingMarker()),
                            ' ', progressbar.ETA(), ' ', progressbar.FileTransferSpeed()]

            self.pbar = progressbar.ProgressBar(widgets=self.widgets, maxval = size).start()

        #print "%d / %d / %d / %d" % (sofar, blocksize, size, sofar*blocksize)
        current = sofar*blocksize
        if current > size:
            self.pbar.update(size)
        else:
            self.pbar.update(sofar * blocksize)

    def stop (self):
        self.pbar.finish()
        self.pbar = None

class TkProgressHook:

    def __init__(self, meter):
        self.meter = meter
        self.text = "Downloading"

    def setText(self, text):
        self.text = text

    def __call__(self, sofar, blocksize, size):
        byteTransferred = sofar*blocksize
        if byteTransferred == 0:
            percentage = 0.0
        else:
            percentage = float(byteTransferred)/float(size)
            #print "%d / %d / %d / %.2f" % (sofar, blocksize, size, percentage)

        text = self.text + " ( %.2f%% )" % (100* percentage)

        self.meter.set(percentage, text)

    def stop(self):
        pass

unfinishedDownload = None

def download(session, downloadDir, ph = ProgressHook()):
    ph.stop()
    trackno = 1
    while True:
        pl = playlist(session)

        for track in pl.playlist.track:
            trackfilename = track["creator"] + " - " + track["album"] + " - " + track["title"] + ".mp3"
            trackfilename = trackfilename.replace("/", "\\/")

            filename = os.path.join(downloadDir, trackfilename)
            ph.setText("#%d Downloading %s" % (trackno, trackfilename))
            global unfinishedDownload
            unfinishedDownload = filename
            interruptedDownload = filename
            try:
                urllib.urlretrieve(track["location"], filename, ph)
            except urllib.ContentTooShortError, e:
                os.remove(interruptedDownload)
                return

            ph.stop()

            try: 
                id3 = mutagen.easyid3.EasyID3(filename)
            except mutagen.id3.ID3NoHeaderError:
                id3 = mutagen.easyid3.EasyID3()

            id3["title"] = track["title"]
            id3["album"] = track["album"]
            id3["artist"] = track["creator"]
            id3.save(filename)
            trackno += 1
